var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var _footer, _form;
import createBoundary from "./util/createBoundary.js";
import getMime from "./util/getMimeFromFilename.js";
import isFormData from "./util/isFormData.js";
import isFile from "./util/isFile.js";
const DASHES = "-".repeat(2);
const CRLF = "\r\n";
const CRLF_BYTES = new TextEncoder().encode(CRLF);
const CRLF_BYTES_LENGTH = CRLF_BYTES.byteLength;
export class Encoder {
    /**
     * Creates a multipart/form-data encoder.
     *
     * @param form - FormData object to encode. This object must be a spec-compatible FormData implementation.
     * @param boundary - An optional boundary string that will be used by the encoder. If there's no boundary string is present, Encoder will generate it automatically.
     *
     * @example
     *
     * import {Encoder} from "form-data-encoder"
     * import {FormData} from "formdata-node"
     *
     * const fd = new FormData()
     *
     * fd.set("greeting", "Hello, World!")
     *
     * const encoder = new Encoder(fd)
     */
    constructor(form, boundary = createBoundary()) {
        /**
         * Returns field's footer
         */
        _footer.set(this, void 0);
        /**
         * FormData instance
         */
        _form.set(this, void 0);
        if (!isFormData(form)) {
            throw new TypeError("Expected first argument to be a FormData instance.");
        }
        if (typeof boundary !== "string") {
            throw new TypeError("Expected boundary to be a string.");
        }
        this.boundary = boundary;
        this.contentType = `multipart/form-data; boundary=${this.boundary}`;
        __classPrivateFieldSet(this, _form, form);
        __classPrivateFieldSet(this, _footer, new TextEncoder().encode(`${DASHES}${this.boundary}${DASHES}${CRLF.repeat(2)}`));
        this.headers = Object.freeze({
            "Content-Type": this.contentType,
            "Content-Length": this.getContentLength()
        });
    }
    _getFieldHeader(name, value) {
        let header = "";
        header += `${DASHES}${this.boundary}${CRLF}`;
        header += `Content-Disposition: form-data; name="${name}"`;
        if (isFile(value)) {
            header += `; filename="${value.name}"${CRLF}`;
            header += `Content-Type: ${value.type || getMime(value.name)}`;
        }
        return new TextEncoder().encode(`${header}${CRLF.repeat(2)}`);
    }
    /**
     * Returns form-data content length
     */
    getContentLength() {
        let length = 0;
        for (const [name, value] of __classPrivateFieldGet(this, _form)) {
            length += this._getFieldHeader(name, value).byteLength;
            length += isFile(value)
                ? value.size
                : new TextEncoder().encode(String(value)).byteLength;
            length += CRLF_BYTES_LENGTH;
        }
        return length + __classPrivateFieldGet(this, _footer).byteLength;
    }
    /**
     * Creates an async iterator allowing to perform the encoding by portions.
     *
     * @example
     *
     * import {Readable} from "stream"
     *
     * import {FormData, File, fileFromPath} from "formdata-node"
     * import {Encoder} from "form-data-encoder"
     *
     * import fetch from "node-fetch"
     *
     * const fd = new FormData()
     *
     * fd.set("field", "Just a random string")
     * fd.set("file", new File(["Using files is class amazing"]))
     * fd.set("fileFromPath", await fileFromPath("path/to/a/file.txt"))
     *
     * const encoder = new Encoder(fd)
     *
     * const options = {
     *   method: "post",
     *   headers: encoder.headers,
     *   body: Readable.from(encoder.encode()) // or Readable.from(encoder)
     * }
     *
     * const response = await fetch("https://httpbin.org/post", options)
     *
     * console.log(await response.json())
     */
    async *encode() {
        for (const [name, value] of __classPrivateFieldGet(this, _form)) {
            yield this._getFieldHeader(name, value);
            if (isFile(value)) {
                yield* value.stream();
            }
            else {
                yield new TextEncoder().encode(String(value));
            }
            yield CRLF_BYTES;
        }
        yield __classPrivateFieldGet(this, _footer);
    }
    [(_footer = new WeakMap(), _form = new WeakMap(), Symbol.asyncIterator)]() {
        return this.encode();
    }
}
